package net.minecraft.src;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import net.minecraft.client.Minecraft;

public class PlayerChat {
	
	static PlayerChat instance;
	static Minecraft mc = ModLoader.getMinecraftInstance(); 
	private final File randomFile, assignedFile;
	public PlayerChat(String fileName){
		if(fileName != null){
			randomFile = new File(mc.getMinecraftDir(), fileName + " Random.txt");
			assignedFile = new File(mc.getMinecraftDir(), fileName + " Assigned.txt");
		}else{
			randomFile = new File(mc.getMinecraftDir(), "Clone Chat Sayings Random.txt");
			assignedFile = new File(mc.getMinecraftDir(), "Clone Chat Sayings Assigned.txt");
		}
		if(!randomFile.exists() || randomFile.length() == 0){
			save(defaultSayings, randomFile);
		}
		if(!assignedFile.exists() || assignedFile.length() == 0){
			save(specialSayings, assignedFile);
		}
		load();
	}
	
	
	
	HashMap<String, String> normalChats = new HashMap<String, String>();
	
	ArrayList<String> randomSayings = new ArrayList<String>();

	public void load(){
		load(normalChats, assignedFile);
		load(randomSayings, randomFile);
	}
	
	public void save(){
		save(normalChats, assignedFile);
		save(randomSayings, randomFile);
	}
	
	public boolean say(String s, EntityMyPerson p){
		if(s != null){
			mc.thePlayer.addChatMessage(("\247" + (teamToColour.get(p.team.teamID)) + p.name +  ": \247" + "f") + (Saying.replace(s)));
			return true;
		}
		return false;
	}
	
	public boolean sayAssigned(String s, EntityMyPerson p){
		if(normalChats.containsKey(s)){
			say(normalChats.get(s), p);
			return true;
		}
		return false;
	}
	
	public boolean remove(String s){
		if(randomSayings.contains(s)){
			randomSayings.remove(s);
			return true;
		}
		return false;
	}
	
	public boolean add(String s){
		if(!randomSayings.contains(s)){
			randomSayings.add(s);
			return true;
		}
		return false;
	}
	
	private static void load(ArrayList<String> random, File rand) {
		if(rand.exists()){
			BufferedReader reader = null;
			try{
				reader = new BufferedReader(new FileReader(rand));
				String s = reader.readLine();
				while(s != null){
					random.add(s);
					s = reader.readLine();
				}
			}catch(Exception e){
				e.printStackTrace();
			}finally{
				if(reader != null){
					try{
						reader.close();
					}catch(Exception e){
						e.printStackTrace();
					}	
				}
			}
		}
	}
	
	private static void load(HashMap<String, String> normal, File norm) {
		if(norm.exists()){
			BufferedReader reader = null;
			try{
				reader = new BufferedReader(new FileReader(norm));
				String s = reader.readLine();
				while(s != null){
					String s2 = reader.readLine();
					if(s2 == null){
						break;
					}
					normal.put(s, s2);
					s = reader.readLine();
				}
			}catch(Exception e){
				e.printStackTrace();
			}finally{
				if(reader != null){
					try{
						reader.close();
					}catch(Exception e){
						e.printStackTrace();
					}	
				}
			}
		}
	}
	
	private static void save(ArrayList<String> random, File rand) {
		if(rand != null && rand.exists()?rand.isDirectory():true){
			BufferedWriter writer = null;
			try{
				writer = new BufferedWriter(new FileWriter(rand));
				for(int a = 0; a < random.size(); a++){
					writer.write(random.get(a) + "\n");
				}
			}catch(Exception e){
				e.printStackTrace();
			}finally{
				if(writer != null){
					try{
						writer.close();
					}catch(Exception e){
						e.printStackTrace();
					}	
				}
			}
		}
	}
	
	private static void save(HashMap<String, String> normal, File norm) {
		if(norm != null && norm.exists()?norm.isDirectory():true){
			BufferedWriter writer = null;
			try{
				writer = new BufferedWriter(new FileWriter(norm));
				for(Iterator<Entry<String, String>> it = normal.entrySet().iterator();it.hasNext();){
					Entry<String, String> entry = it.next();
					writer.write(entry.getKey() + "\n");
					writer.write(entry.getValue() + "\n");
				}
			}catch(Exception e){
				e.printStackTrace();
			}finally{
				if(writer != null){
					try{
						writer.close();
					}catch(Exception e){
						e.printStackTrace();
					}	
				}
			}
		}
	}
	
	static ArrayList<String> defaultSayings = new ArrayList<String>();
	static HashMap<String, String> specialSayings = new HashMap<String, String>();
	static HashMap<Integer, String> teamToColour = new HashMap<Integer, String>();
	static{
		for(int a = 0; a < Saying.sayings.size(); a++){
			specialSayings.put(Saying.sayings.get(a).key, Saying.sayings.get(a).value);
		}
		instance = new PlayerChat("Clone Chat Sayings");
		teamToColour.put(0, "c");
		teamToColour.put(1, "6");
		teamToColour.put(2, "e");
		teamToColour.put(3, "a");
		teamToColour.put(4, "1");
		teamToColour.put(5, "3");
		teamToColour.put(6, "5");
		teamToColour.put(7, "d");
		teamToColour.put(8, "f");
		teamToColour.put(9, "8");
		teamToColour.put(10, "0");
	}
	//
	public static class Saying{
		static String replace = "\246";
		
		static ArrayList<Saying> sayings = new ArrayList<Saying>();
		static HashMap<String, String> sayings2 = new HashMap<String, String>();
		
		String key = null;
		String value = null;
		
		public Saying(String s, String s2){
			key = s;
			value = s2;
			sayings.add(this);
			sayings2.put(s, s2);
		}
		
		public static String get(String s){
			if(s != null && sayings2.containsKey(s)){
				return replace(sayings2.get(s));
			}
			return replace(s);
		}
		
		public static String get(Saying s){
			if(s != null){
				return replace(s.value);
			}
			return "";
		}
		
		public static String replace(String s){
			if(s != null && s.contains(replace)){
				return s.replace(replace, mc.thePlayer.username);
			}
			return s;
		}
		static Saying born, die, mine, hurt, runAttack, runCreeper, onFire, offFire, falling, drown, shot, pickup, breaka, hungry;
		static{
			born = new Saying("born", "Gee that egg is cramped.");
			die = new Saying("die", "NOOO! I'm dying, and I can't get up... *Cough* *Cough*... *dies*");
			mine = new Saying("mine", "Breakedy break!");
			hurt = new Saying("hurt", "Ouch!");
			runAttack = new Saying("runAttack", "I'm gonna getcha!");
			runCreeper = new Saying("runCreeper", "Ooh; gee; run!");
			onFire = new Saying("onFire", "UHH!?!?! A BIT OF HELP PLEASE " + replace + "!");
			offFire = new Saying("offFire", "Ah, that's a bit cooler...");
			falling = new Saying("falling", "Wahoooooo!!!");
			drown = new Saying("drown", "Umm... " + replace + "! I'm drowning! Arghhhh!");
			shot = new Saying("shot", "Hey! That's no fair! Bow's are cheating!");
			pickup = new Saying("pickup", "Hey, what's this?");
			breaka = new Saying("break", "Damn, I needed that!");
			hungry = new Saying("hungry", "Gee I'm hungry... I really need some food " + replace);
		}
	}
	public boolean sayRandom(EntityMyPerson p) {
		if(randomSayings.size() != 0){
			say(randomSayings.get(p.worldObj.rand.nextInt(randomSayings.size())), p);
		}
		return false;
		
	}

}
