package net.minecraft.src;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

import javax.imageio.ImageIO;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.Minecraft;
import paulscode.sound.SoundSystem;

public class SoundPackage {
	
	HashMap<String, List<SoundPoolEntry>> stringToSoundList = new HashMap<String, List<SoundPoolEntry>>();
	File soundDir = null;
	BufferedImage iconImage;
	String iconLoc;
	String packName;
	String packText;
	String packAuthor = "Unknown";
	public SoundPackage(File f){
		soundDir = f;
		packText = f.getName();
		packName = f.getName();
		File[] files = f.listFiles();
		for(int a = 0; a < files.length; a++){
			if(files[a] != null && files[a].exists() && isValidSoundFile(files[a])){
				addSound(files[a]);
			}else if(files[a] != null && files[a].exists() && files[a].getName().toLowerCase().equals("pack.txt")){
				readPackInfo(files[a]);
			}else if(files[a] != null && files[a].exists() && files[a].getName().toLowerCase().startsWith("icon.")){
				readPackImage(files[a]);
			}
		}
	}
	
	private void readPackImage(File file) {
		try {
			BufferedImage image = ImageIO.read(file);
			iconImage = new BufferedImage(64, 64, BufferedImage.TYPE_INT_ARGB);
			Graphics g = iconImage.getGraphics();
			g.drawImage(image, 0, 0, 64, 64, 0, 0, image.getWidth(), image.getHeight(), null);
			g.dispose();
			iconLoc = file.getAbsolutePath();
			if(iconImage != null && !iconToTexture.containsKey(file.getAbsolutePath())){
				iconToTexture.put(file.getAbsolutePath(), mc.renderEngine.allocateAndSetupTexture(iconImage));
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	private void readPackInfo(File file) {
		
		BufferedReader in = null;
		try{
			in = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
			String s1 = in.readLine();
			if(s1 != null){
				packText = s1;
			}
			s1 = in.readLine();
			if(s1 != null){
				packAuthor = s1;
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			if(in != null){
				try{
					in.close();
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		}
	}

	private boolean isValidSoundFile(File file) {
		String s = file.getName();
		s = s.substring(s.lastIndexOf(".")).toLowerCase();
		return s.equals(".ogg") || s.equals(".mus") || s.equals(".wav");
	}
	
	public void addSound(File f){

		String s = f.getName();
		if(s.contains(".")){
			s = s.substring(0, s.lastIndexOf("."));
		}
		
		for(;Character.isDigit(s.charAt(s.length()-1));s = s.substring(0, s.length()-1)){}
		if(!stringToSoundList.containsKey(s)){
			stringToSoundList.put(s, new ArrayList());
		}
		try{
			stringToSoundList.get(s).add(new SoundPoolEntry(f.getName(), f.toURI().toURL()));
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public SoundPoolEntry getRandom(String s){
		if(stringToSoundList.containsKey(s)){
			return getRandom(stringToSoundList.get(s));
		}
		return null;
	}
	
	public SoundPoolEntry getRandom(List<SoundPoolEntry> list){
		return list.get(rand.nextInt(list.size()));
	}
    /**
     * Plays a sound. Args: soundName, x, y, z, volume, pitch
     */
/*    public boolean playSound(String par1Str, float par2, float par3, float par4, float par5, float par6)
    {
    	getSnd();
        if (mc.gameSettings.soundVolume == 0.0F)
        {
            return false;
        }

        SoundPoolEntry soundpoolentry = getRandom(par1Str);
        if (soundpoolentry != null && par5 > 0.0F)
        {
            latestSoundID = ((latestSoundID + 1) % 256);
            String s = (new StringBuilder()).append("sound_").append((latestSoundID + 256)).toString();
            float f = 16F;

            if (par5 > 1.0F)
            {
                f *= par5;
            }

            sndSystem.newSource(par5 > 1.0F, s, soundpoolentry.soundUrl, soundpoolentry.soundName, false, par2, par3, par4, 2, f);
            sndSystem.setPitch(s, par6);

            if (par5 > 1.0F)
            {
                par5 = 1.0F;
            }

            sndSystem.setVolume(s, par5 * mc.gameSettings.soundVolume);
            sndSystem.play(s);
           	return true;
        }
        return false;
    }*/
    public boolean play(String par1Str, float par2, float par3, float par4, float par5, float par6, EntityMyPerson p)
    {
    	getSnd();
        if (mc.gameSettings.soundVolume == 0.0F)
        {
            return false;
        }

        SoundPoolEntry soundpoolentry = getRandom(par1Str);
        if (soundpoolentry != null && par5 > 0.0F)
        {
            latestSoundID = ((latestSoundID + 1) % 256);
            String s = (new StringBuilder()).append("sound_").append(par1Str).append("").toString();
            
            if(sndSystem.playing(s)){
            	System.out.println("stop");
            	sndSystem.stop(s);
            }
            float f = 16F;

            if (par5 > 1.0F)
            {
                f *= par5;
            }

            sndSystem.newSource(par5 > 1.0F, s, soundpoolentry.soundUrl, soundpoolentry.soundName, false, par2, par3, par4, 2, f);
            sndSystem.setPitch(s, par6);

            if (par5 > 1.0F)
            {
                par5 = 1.0F;
            }

            sndSystem.setVolume(s, par5 * mc.gameSettings.soundVolume);
            sndSystem.play(s);
           	return true;
        }
        return false;
    }

	public boolean play2(String par1Str, double posX, double posY,
			double posZ, float par5, float par6, EntityMyPerson p) {
		return play(par1Str, (float)posX, (float)posY, (float)posZ, par5, par6, p);
	}
	
	public static boolean play1(String par1Str, double posX, double posY, double posZ, float par5, float par6, EntityMyPerson p){
		if(currentSelected < allPackages.size() && allPackages.get(currentSelected) != null){
			return allPackages.get(currentSelected).play2(par1Str, posX, posY, posZ, par5, par6, p);
		}
		return false;
	}
    
	static SoundSystem sndSystem;
	static Random rand = new Random();
	static Minecraft mc = ModLoader.getMinecraftInstance();
	static ArrayList<SoundPackage> allPackages = new ArrayList<SoundPackage>();
	static HashMap<String, SoundPackage> stringToPackage = new HashMap<String, SoundPackage>();
	static HashMap<SoundPackage, String> packageToString = new HashMap<SoundPackage, String>();
	static HashMap<String, Integer> iconToTexture = new HashMap<String, Integer>();
	static int latestSoundID = 0;
	static File globalSoundDir;
	
	static int currentSelected = 0;
	static String currentSelectedS;
	
	public static void loadPackages(){
		stringToPackage.clear();
		allPackages.clear();
		globalSoundDir = new File(ModLoader.getMinecraftInstance().getMinecraftDir(), "MyPeopleSoundPacks");
		if(!globalSoundDir.exists()){
			globalSoundDir.mkdir();
		}
		File[] fs = globalSoundDir.listFiles();
		for(int a = 0; a < fs.length; a++){
			if(fs[a].isDirectory()){
				SoundPackage p = new SoundPackage(fs[a]);
				stringToPackage.put(fs[a].getName(), p);
				packageToString.put(p, fs[a].getName());
				allPackages.add(p);
			}
		}
	}
	
	private static SoundSystem getSnd() {
		if(sndSystem == null){
			Field[] fields = SoundManager.class.getDeclaredFields();
			try{
				for(int a = 0; a < fields.length; a++){
					if(fields[a] != null && fields[a].getType().equals(SoundSystem.class)){
						fields[a].setAccessible(true);
						sndSystem = (SoundSystem)fields[a].get(null);
						return sndSystem;
					}
				}
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		return sndSystem;
	}
	
	public static void init(){}

	static{
		loadPackages();
		load();
		getSnd();
	}
	
	public static boolean setSelected(String s, boolean save){
		if(stringToPackage.containsKey(s)){
			currentSelected = allPackages.indexOf(stringToPackage.get(s));
			currentSelectedS = s;
			if(save){
				save();
			}
			return true;
		}
		return false;
	}
	
	public static boolean setSelected(int i, boolean save){
		if(i < allPackages.size()){
			currentSelected = i;
			currentSelectedS = packageToString.get(allPackages.get(i));
			if(save){
				save();
			}
			return true;
		}
		return false;
	}
	
	public static void save(){
		File f = new File(globalSoundDir, "Selected.dat");
		DataOutputStream out = null;
		try{
			out = new DataOutputStream(new FileOutputStream(f));
			out.writeUTF(currentSelectedS);
			out.writeInt(currentSelected);
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			if(out != null){
				try{
					out.close();
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		}
	}
	
	public static void load(){
		File f = new File(globalSoundDir, "Selected.dat");
		if(!f.exists()){
			try {
				f.createNewFile();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		DataInputStream in = null;
		try{
			in = new DataInputStream(new FileInputStream(f));
			if(!setSelected(in.readUTF(), false)){
				setSelected(in.readInt(), false);
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			if(in != null){
				try{
					in.close();
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		}
	}
	
	public void bindIcon(){
		if(iconLoc != null && iconToTexture.containsKey(iconLoc)){
			mc.renderEngine.bindTexture(iconToTexture.get(iconLoc));
		}else{
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, mc.renderEngine.getTexture("/gui/unknown_pack.png"));
		}
	}
}
